/* -*- c++ -*-
 * 
 * Copyright (c) 2007 Anthony Mallet                  --  Sat Mar  3 2007
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
#ifndef H_R3D
#define H_R3D

#include "t3d/t3d.h"

/* --- API ----------------------------------------------------------- */

/** \addtogroup C */
/** \{ */

extern const R3D r3dBryanIdentity;	/**<\brief static initializer. */
extern const R3D r3dUBryanIdentity;	/**<\brief static initializer. */
extern const R3D r3dVectorIdentity;	/**<\brief static initializer. */
extern const R3D r3dUVectorIdentity;	/**<\brief static initializer. */
extern const R3D r3dMatrixIdentity;	/**<\brief static initializer. */
extern const R3D r3dUMatrixIdentity;	/**<\brief static initializer. */

#ifdef __cplusplus
extern "C" {
#endif

  R3D *		r3dNew(T3DType type, int flags);
  R3D *		r3dInit(R3D *r3d, T3DType type, int flags);
  R3D *		r3dDup(R3D *r3d);
  void		r3dDelete(R3D *r3d);

  T3D *		r3dConvertToT3D(R3D *r3d);
  R3D *		r3dConvertTo(T3DType typeTo, R3D *r3d);
  R3D *		r3dCopyIn(R3D *cpy, const R3D *src);
  R3D *		r3dInvert(R3D *r3d);

  R3D *		r3dComp(R3D *a, R3D *b);
  R3D *		r3dCompIn(R3D *c, R3D *a, R3D *b);

  int		r3dFPrint(FILE *stream, const R3D *r3d);

  static inline void	r3dWarn(const R3D *r3d);
  static inline int	r3dOk(const R3D *r3d);
  static inline void	r3dSetOk(R3D *r3d);
  static inline R3D *	r3dSetError(R3D *r3d, int err);

#ifdef __cplusplus
}
#endif

/** \} */


/* --- R3D data structures ------------------------------------------- */

/** \brief A 3D rotation
 *
 * This union is the base class of rotations. It can hold
 * any representation, but only one at a time. Implicits or explicits
 * conversions can be performed.
 */
union R3D {
#ifdef __cplusplus
 public:
  inline R3D(T3DType type, int flags, T3DError error,const double* d1 = 0x0,const double* d2 = 0x0);
  inline R3D(T3DType type = T3D_MATRIX, int flags = 0);
  inline R3D(const R3D *r3d);
  inline R3D(const T3DMatrix &m);

  inline R3D &invert(void);
  inline R3D &convertTo(T3DType type);

  inline R3D operator~(void);
  inline R3D operator*(R3D &r3d);
  friend inline V3D operator*(R3D &r3d, const V3D &b);

  friend std::ostream &operator<<(std::ostream &s, const R3D &r3d);
#endif

  struct /* anonymous */ { _T3DHeader_members; };

  T3DMatrix	matrix;		/**<\brief Homogeneous matrix. */
  T3DUMatrix	umatrix;	/**<\brief Uncertain homogeneous matrix. */
  T3DBryan	bryan;		/**<\brief Tait-Bryan angles and translation. */
  T3DUBryan	ubryan;		/**<\brief Uncertain Tait-Bryan. */
  T3DVector	vector;		/**<\brief Rotation vector. */
  T3DUVector	uvector;	/**<\brief Uncertain rotation vector. */
};

/* include inlined functions and methods */
# include "t3d/r3d_inline.h"

#endif /* H_R3D */
